# Generated by Django 5.1.6 on 2025-02-12 10:56

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='TestModelRelated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('test_model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app.testmodel')),
            ],
        ),
    ]



# serializer_generator.py
from django_model_suite.generators.base import BaseGenerator

class SerializerGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"
        content = f"""from rest_framework import serializers
from {model_import_path} import {model_name}

class {model_name}MinimalSerializer(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = (id, )
        read_only = True

class {model_name}DetailedSerializer(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = (id,)
        read_only = True

class {model_name}CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = ()

class {model_name}UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = ()
"""
        self.write_file("serializers.py", content)


# permissions_generator.py
from typing import List

from ..base import BaseGenerator


class PermissionsGenerator(BaseGenerator):
    def generate(self, fields: List[str]) -> None:
        model_name = self.model.__name__
        field_rules = [
            f"            {model_name}Fields.{field.upper()}: FieldPermissions(\n"
            "                visible=(\n"
            "                    context.is_superuser\n"
            "                ),\n"
            "                editable=(\n"
            "                ),\n"
            "            )" for field in fields
        ]

        content = f"""from typing import Optional, Dict
from django.http import HttpRequest
from ...fields.{self.model_name_lower} import {model_name}Fields
from django_model_suite.admin import FieldPermissions
from {self.model.__module__} import {model_name}
from .context import {model_name}ContextLogic


class {model_name}Permissions:
    def has_add_permission(self, request: HttpRequest) -> bool:
        return False

    def has_change_permission(self, request: HttpRequest, {self.model_name_lower}: Optional[{model_name}] = None) -> bool:
        return False

    def has_delete_permission(self, request: HttpRequest, {self.model_name_lower}: Optional[{model_name}] = None) -> bool:
        return False

    def get_field_rules(self, request: HttpRequest, {self.model_name_lower}: Optional[{model_name}] = None) -> Dict:
        context = {model_name}ContextLogic(request, {self.model_name_lower})

        return {{
""" + ',\n'.join(field_rules) + """
        }
"""
        self.write_file('permissions.py', content)


# service_generator.py
from ..base import BaseGenerator

class ServiceGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"
        content = f'''from typing import Any
from {model_import_path} import {model_name}
from ..validators.{self.model_name_lower} import validate_{self.model_name_lower}_create, validate_{self.model_name_lower}_update

class {model_name}Service:
    @staticmethod
    def create_{self.model_name_lower}(*, data: dict[str, Any]) -> {model_name}:
        validated_data = validate_{self.model_name_lower}_create(data=data)
        instance = {model_name}.objects.create(**validated_data)
        return instance

    @staticmethod
    def update_{self.model_name_lower}(*, instance: {model_name}, data: dict[str, Any]) -> {model_name}:
        validated_data = validate_{self.model_name_lower}_update(instance=instance, data=data)
        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()
        return instance

    @staticmethod
    def delete_{self.model_name_lower}(*, instance: {model_name}) -> None:
        instance.delete()
'''
        self.write_file(f'{self.model_name_lower}.py', content)


from django.apps import AppConfig


class AppConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'app'

    # def ready(self):
    #     from app.admin.testmodelrelated import admin  # noqa



# validator_generator.py
from ..base import BaseGenerator

class ValidatorGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"
        content = f'''from typing import Any
from django.core.exceptions import ValidationError
from {model_import_path} import {model_name}

def validate_{self.model_name_lower}_create(*, data: dict[str, Any]) -> dict[str, Any]:
    # Add your validation logic here
    return data

def validate_{self.model_name_lower}_update(*, instance: {model_name}, data: dict[str, Any]) -> dict[str, Any]:
    # Add your validation logic here
    return data
'''
        self.write_file(f'{self.model_name_lower}.py', content)


# context_generator.py
from typing import List
from ..base import BaseGenerator

class ContextGenerator(BaseGenerator):
    def generate(self, fields: List[str] = None) -> None:
        model_name = self.model.__name__
        content = f"""from typing import Optional
from django.http import HttpRequest
from {self.model.__module__} import {model_name}


class {model_name}ContextLogic:
    def __init__(self, request: HttpRequest, {self.model_name_lower}: Optional[{model_name}] = None):
        self.request = request
        self.{self.model_name_lower} = {self.model_name_lower}

    @property
    def is_superuser(self) -> bool:
        return self.request.user.is_superuser

    @property
    def is_staff(self) -> bool:
        return self.request.user.is_staff

    @property
    def is_creating(self) -> bool:
        return self.{self.model_name_lower} is None
"""
        self.write_file('context.py', content)


# selector_generator.py
from ..base import BaseGenerator

class SelectorGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"
        content = f'''from django.db.models import QuerySet
from {model_import_path} import {model_name}

class {model_name}Selector:
    @staticmethod
    def get_queryset() -> QuerySet:
        return {model_name}.objects.all()

    @staticmethod
    def by_id(*, id: int) -> {model_name}:
        return {model_name}Selector.get_queryset().get(id=id)
'''
        self.write_file(f'{self.model_name_lower}.py', content)


import os

from django.apps import apps


def get_model_fields(app_name: str, model_name: str) -> list:
    """Get direct fields and foreign keys from a model."""
    model = apps.get_model(app_name, model_name)
    return [
        field.name
        for field in model._meta.get_fields()
        if not field.auto_created
    ]
def ensure_package(path: str) -> None:
    os.makedirs(path, exist_ok=True)
    init_file = os.path.join(path, "__init__.py")
    if not os.path.exists(init_file):
        with open(init_file, "w"):
            pass



# display_generator.py
from ..base import BaseGenerator

class DisplayGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"
        content = f'''from unfold.decorators import display
from {model_import_path} import {model_name}

class {model_name}DisplayMixin:
    @display(description="{self.model_name_lower}", header=True)
    def display_header(self, {self.model_name_lower}: {model_name}):
        """Display header with image if available."""
        return [
            {self.model_name_lower}.pk,
            "",
            "O",
            {{"path": {self.model_name_lower}.image.url if hasattr({self.model_name_lower}, 'image') and {self.model_name_lower}.image else None}},
        ]
'''
        self.write_file('display.py', content)


from ..base import BaseGenerator


class AdminGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        model_name = self.model.__name__
        model_import_path = f"{self.model.__module__}"

        content = f'''from django.contrib import admin
from .list_view import {model_name}ListView
from .change_view import {model_name}ChangeView
from .permissions import {model_name}Permissions
from .display import {model_name}DisplayMixin
from {model_import_path} import {model_name}
from django_model_suite.admin import BaseModelAdmin

@admin.register({model_name})
class {model_name}Admin(
    {model_name}DisplayMixin,
    {model_name}ListView,
    {model_name}ChangeView,
    {model_name}Permissions,
    BaseModelAdmin,
):
    pass
'''
        self.write_file('admin.py', content)


import os
import re
from abc import ABC, abstractmethod

from django_model_suite.generators.model_utils import ensure_package


class BaseGenerator(ABC):
    def __init__(self, app_name: str, model_name: str, base_path: str, model_class):
        self.base_path = base_path
        self.model = model_class
        self.model_name_lower = self._to_snake_case(self.model.__name__)

    def _to_snake_case(self, name: str) -> str:
        """Convert CamelCase to snake_case (e.g., TestModelRelated -> test_model_related)"""
        pattern = re.compile(r'(?<!^)(?=[A-Z])')

        return pattern.sub('_', name).lower()

    def write_file(self, filename: str, content: str) -> None:
        target_dir = self.base_path
        full_path = os.path.join(target_dir, filename)

        ensure_package(target_dir)
        if os.path.exists(full_path):
            return

        with open(full_path, "w") as f:
            f.write(content)

    @abstractmethod
    def generate(self, fields: list) -> None:
        pass


from django.contrib import admin
from unfold.admin import ModelAdmin

from app.models import TestModel


@admin.register(TestModel)
class AuthorAdmin(ModelAdmin):
    list_display = ('name',)

#
# @admin.register(TestModelRelated)
# class TestModelRelatedAdmin(ModelAdmin):
#     list_display = ('test_model', 'name')



from django.db import models


class TestModel(models.Model):
    name = models.CharField(max_length=100)


class TestModelRelated(models.Model):
    test_model = models.ForeignKey(TestModel, on_delete=models.CASCADE)
    name = models.CharField(max_length=100)


class UnRelatedModel(models.Model):
    name = models.CharField(max_length=100)
    test_model = models.ForeignKey(TestModel, on_delete=models.CASCADE)
    test_model_related = models.ForeignKey(TestModelRelated, on_delete=models.CASCADE)
